/* logview-about.h - authors and contributors information
 *
 * Copyright (C) 2004 Vincent Noel <vnoel@cox.net>
 * Copyright (C) 2008 Cosimo Cecchi <cosimoc@gnome.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

#ifndef __LOGVIEW_ABOUT_H__
#define __LOGVIEW_ABOUT_H__

static const char *logview_about_authors[] = {
  "Cesar Miquel <miquel@df.uba.ar>",
  "Glynn Foster <glynn.foster@sun.com>",
  "Fernando Herrera  <fherrera@onirica.com>",
  "Shakti Sen  <shprasad@novell.com>",
  "Julio M Merino Vidal <jmmv@menta.net>",
  "Jason Leach  <leach@wam.umd.edu>",
  "Christian Neumair  <chris@mate-de.org>",
  "Jan Arne Petersen  <jpetersen@uni-bonn.de>",
  "Jason Long  <jlong@messiah.edu>",
  "Kjartan Maraas  <kmaraas@gnome.org>",
  "Vincent Noel  <vincent.noel@gmail.com>",
  "Cosimo Cecchi <cosimoc@gnome.org>",
  NULL
};

static const char *logview_about_documenters[] = {
  "Sun MATE Documentation Team <gdocteam@sun.com>",
  "Vincent Noel <vincent.noel@gmail.com>",
  "Judith Samson <judith@samsonsource.com>",
  NULL
};

static const char * logview_about_license[] = {
  N_("This program is free software; you can redistribute it and/or modify "
     "it under the terms of the GNU General Public License as published by "
     "the Free Software Foundation; either version 2 of the License, or "
     "(at your option) any later version."),
  N_("This program is distributed in the hope that it will be useful, "
     "but WITHOUT ANY WARRANTY; without even the implied warranty of "
     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
     "GNU General Public License for more details."),
  N_("You should have received a copy of the GNU General Public License "
     "along with this program; if not, write to the Free Software Foundation, Inc., "
     "51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA")
};

/* translator credits */
static const char *logview_about_translator_credits = N_("translator-credits");


#endif /* __LOGVIEW_ABOUT_H__ */

